import { useAppSelector } from '@/redux/hooks/useAppSelector';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import {
  layersActiveLayersSelector,
  layersVisibilityForCurrentModelSelector,
} from '@/redux/layers/layers.selectors';
import { currentModelIdSelector } from '@/redux/models/models.selectors';
import { openLayerFactoryModal } from '@/redux/modal/modal.slice';
import QuestionModal from '@/components/FunctionalArea/Modal/QuestionModal/QustionModal.component';
import { useState, JSX, useMemo } from 'react';
import { getLayersForModel, removeLayer } from '@/redux/layers/layers.thunks';
import { showToast } from '@/utils/showToast';
import { SerializedError } from '@reduxjs/toolkit';
import { LayersDrawerLayerActions } from '@/components/Map/Drawer/LayersDrawer/LayersDrawerLayerActions.component';
import {
  setDrawLayer,
  setLayerToActive,
  setLayerToInactive,
  setLayerVisibility,
} from '@/redux/layers/layers.slice';
import { LayersDrawerObjectsList } from '@/components/Map/Drawer/LayersDrawer/LayersDrawerObjectsList.component';
import { mapEditToolsSetActiveAction } from '@/redux/mapEditTools/mapEditTools.slice';
import { MAP_EDIT_ACTIONS } from '@/redux/mapEditTools/mapEditTools.constants';

interface LayersDrawerLayerProps {
  layerId: number;
  layerName: string;
}

export const LayersDrawerLayer = ({ layerId, layerName }: LayersDrawerLayerProps): JSX.Element => {
  const layersVisibilityForCurrentModel = useAppSelector(layersVisibilityForCurrentModelSelector);
  const activeLayers = useAppSelector(layersActiveLayersSelector);
  const currentModelId = useAppSelector(currentModelIdSelector);
  const dispatch = useAppDispatch();
  const [isModalOpen, setIsModalOpen] = useState(false);

  const isLayerVisible = useMemo(() => {
    return layersVisibilityForCurrentModel[layerId];
  }, [layerId, layersVisibilityForCurrentModel]);

  const isLayerActive = useMemo(() => {
    return activeLayers.includes(layerId);
  }, [activeLayers, layerId]);

  const editLayer = (): void => {
    dispatch(openLayerFactoryModal(layerId));
  };

  const addImage = (): void => {
    dispatch(setDrawLayer({ modelId: currentModelId, layerId }));
    dispatch(mapEditToolsSetActiveAction(MAP_EDIT_ACTIONS.DRAW_IMAGE));
  };

  const addText = (): void => {
    dispatch(setDrawLayer({ modelId: currentModelId, layerId }));
    dispatch(mapEditToolsSetActiveAction(MAP_EDIT_ACTIONS.ADD_TEXT));
  };

  const rejectRemove = (): void => {
    setIsModalOpen(false);
  };

  const confirmRemove = async (): Promise<void> => {
    if (!layerId) {
      return;
    }
    try {
      await dispatch(removeLayer({ modelId: currentModelId, layerId })).unwrap();
      showToast({
        type: 'success',
        message: 'The layer has been successfully removed',
      });
      setIsModalOpen(false);
      dispatch(getLayersForModel(currentModelId));
    } catch (error) {
      const typedError = error as SerializedError;
      showToast({
        type: 'error',
        message: typedError.message || 'An error occurred while removing the layer',
      });
    }
  };

  const onRemoveLayer = (): void => {
    setIsModalOpen(true);
  };

  const toggleActiveLayer = (value: boolean): void => {
    if (value) {
      dispatch(setLayerToActive({ modelId: currentModelId, layerId }));
    } else {
      dispatch(setLayerToInactive({ modelId: currentModelId, layerId }));
    }
  };

  return (
    <div>
      <QuestionModal
        isOpen={isModalOpen}
        onClose={rejectRemove}
        onConfirm={confirmRemove}
        question="Are you sure you want to remove the layer?"
      />
      <div className="flex items-center justify-between py-3">
        <span className={`font-semibold ${isLayerVisible ? 'opacity-100' : 'opacity-40'}`}>
          {layerName}
        </span>
        <LayersDrawerLayerActions
          toggleVisibility={() =>
            dispatch(
              setLayerVisibility({
                modelId: currentModelId,
                visible: !isLayerVisible,
                layerId,
              }),
            )
          }
          toggleActiveLayer={toggleActiveLayer}
          editLayer={editLayer}
          removeLayer={onRemoveLayer}
          addImage={addImage}
          addText={addText}
          isVisible={isLayerVisible}
          isActive={isLayerActive}
        />
      </div>
      <LayersDrawerObjectsList
        layerId={layerId}
        isLayerVisible={isLayerVisible}
        isLayerActive={isLayerActive}
      />
    </div>
  );
};
